@@ -17,7 +17,8 @@
 ######################################################################
 "update.statnet" <- function(object, ..., 
             contriburl = "http://statnetproject.org",
-            repos = getOption("repos"), type = getOption("pkgType"))
+            repos = getOption("repos"), type = getOption("pkgType"),
+            ask=TRUE)
 {
   cran.contriburl <- contrib.url(repos, type)
   if(contriburl %in% c("http://csde.washington.edu/statnet",
@@ -46,16 +47,25 @@
 #
 # local install functions
 #
-  really.detach <- function(package){
-   inuse <- match(paste("package:",pkg,sep=""), search())
-   if(!is.na(inuse) && !inherits(try(detach(pos=inuse, unload=TRUE),silent=TRUE), "try-error")){
-     cat(paste("Detaching package '", pkg,"'.\n",sep=""))
-     inuse <-  grep(paste("/",package,"$",sep=""),searchpaths())
-     if(length(inuse)>0){
-      inuse <- searchpaths()[inuse]
-      library.dynam.unload(paste(package,".so",sep=""),inuse)
-     }
+  really.detach <- function(packages, verbose=TRUE){
+   for(pkg in packages){
+    inuse <- match(paste("package:",pkg,sep=""), search())
+    if(!is.na(inuse)){
+     silentwarnings <- capture.output(
+      try(detach(pos=inuse, unload=TRUE),silent=TRUE)
+                                     )
+     if(!inherits(silentwarnings, "try-error")){
+      if(verbose){
+       cat(paste("Detaching package '", pkg,"'.\n",sep=""))
+      }
+      inuse <-  grep(paste("/",pkg,"$",sep=""),searchpaths())
+      if(length(inuse)>0){
+       inuse <- searchpaths()[inuse]
+       library.dynam.unload(paste(pkg,".so",sep=""),inuse)
+      }
+    }
    }
+  }
   invisible()
   }
   statnet.install <- function(object, csde, contriburl=cran.contriburl,
@@ -67,11 +77,7 @@
       cat(paste("=========================================\n",sep=""))
       if(pkg %in% update.pkgs){
        cat(paste("Attempting to update package '", pkg,"'.\n",sep=""))
-       inuse <- match(paste("package:",pkg,sep=""), search())
-       if(!is.na(inuse)){
-         if(!inherits(try(detach(pos=inuse, unload=TRUE),silent=TRUE), "try-error")){
-           cat(paste("Detaching package '", pkg,"'.\n",sep=""))}
-       }
+       really.detach(pkg)
        if(ask){
         if(type == "recommended"){
          cat(paste("It is recommended you update '",pkg,"'.\n", sep=""))
@@ -93,7 +99,7 @@
   invisible()
   }
 # Detach the packages to avoid conflicts
-  for(pkg in object){ really.detach(pkg) }
+  really.detach(object)
 # CSDE
   new.csde.pkgs <- new.packages(lib.loc=.libPaths()[1],contriburl=contriburl)
   old.csde.pkgs <- old.packages(lib.loc=.libPaths()[1],contriburl=contriburl)[,1]
@@ -103,15 +109,24 @@
   old.cran.pkgs <- old.packages(lib.loc=.libPaths()[1])[,1]
   update.cran.pkgs <- c(new.cran.pkgs, old.cran.pkgs)
 #
-  answer<-substr(readline(paste(" Install and update all packages (y)\n   or select individual packages (n)?  ", sep="")),1,1)
+  if(ask){
+   answer<-substr(readline(paste(" Install and update all packages (y)\n   or select individual packages (n)?  ", sep="")),1,1)
+  }else{
+   answer<-TRUE
+  }
   if (answer == "y" | answer == "Y" | answer == ""){
    statnet.install(object, cran.Base, ask=FALSE, type="base")
+   really.detach(object)
    statnet.install(object, csde.Base, ask=FALSE, type="base",
                    update.pkgs=update.csde.pkgs, contriburl=contriburl)
+   really.detach(object)
    statnet.install(object, cran.Recommended, ask=FALSE, type="recommended")
+   really.detach(object)
    statnet.install(object, csde.Recommended, ask=FALSE, type="recommended",
                    update.pkgs=update.csde.pkgs, contriburl=contriburl)
+   really.detach(object)
    statnet.install(object, cran.Optional, ask=FALSE, type="optional")
+   really.detach(object)
    if( 'rSoNIA' %in% csde.Optional &&  
       !('dynamicnetwork' %in% (installed.packages())[,"Package"])){
       cat("\n\n'rSoNIA' requires 'dynamicnetwork' to be installed manually before it can be installed. 'rSoNIA' has not been installed.\n Instructions for downloading and installing a working system are located at 'http://csde.washington.edu/~skyebend/installDynamicnetwork.html'\n\n\n")
@@ -119,14 +134,20 @@
    }
    statnet.install(object, csde.Optional, ask=FALSE, type="optional",
                    update.pkgs=update.csde.pkgs, contriburl=contriburl)
+   really.detach(object)
   }else{
    statnet.install(object, cran.Base, ask=FALSE, type="base")
+   really.detach(object)
    statnet.install(object, csde.Base, ask=FALSE, type="base",
                    update.pkgs=update.csde.pkgs, contriburl=contriburl)
+   really.detach(object)
    statnet.install(object, cran.Recommended, ask=TRUE, type="recommended")
+   really.detach(object)
    statnet.install(object, csde.Recommended, ask=TRUE, type="recommended",
                    update.pkgs=update.csde.pkgs, contriburl=contriburl)
+   really.detach(object)
    statnet.install(object, cran.Optional, ask=TRUE, type="optional")
+   really.detach(object)
    if( 'rSoNIA' %in% csde.Optional &&  
       !('dynamicnetwork' %in% (installed.packages())[,"Package"])){
       cat("\n\n'rSoNIA' requires 'dynamicnetwork' to be installed manually before it can be installed. 'rSoNIA' has not been installed.\n Instructions for downloading and installing a working system are located at 'http://csde.washington.edu/~skyebend/installDynamicnetwork.html'\n\n\n")
@@ -134,6 +155,7 @@
    }
    statnet.install(object, csde.Optional, ask=TRUE, type="optional",
                    update.pkgs=update.csde.pkgs, contriburl=contriburl)
+   really.detach(object)
   }
 #
 # check required packages for latentnet
@@ -141,6 +163,7 @@
   inst.pkgs <- as.vector(installed.packages()[,1])
   if("latentnet" %in% inst.pkgs){
     statnet.install(cran.Base.latentnet, cran.Base.latentnet, ask=FALSE, type="base")
+    really.detach(object)
   }
 #
   cat(paste("=========================================\n",sep=""))
